#include <string.h>
#include "mex.h"
#include "matrix.h"

#define element(rows, row, col) (rows*col + row)
#define MAX(a,b) ((a>b)?a:b)
#define MIN(a,b) ((a<b)?a:b)

  
/*
 * C = bprod(A, B)
 *
 * Returns the Boolean product of A and B.
 *
 */ 

void 
mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  mwSize n, m, k, foo, i, j, l;
  double *A, *B, *C;


  /* Check for input and output argument numbers */
  if (nrhs != 2) 
    mexErrMsgTxt("Two input arguments, A and B, required.");
  if (nlhs > 1)
    mexErrMsgTxt("Too many output arguments.");
  
  /* Check and load input matrices. */
  A = mxGetPr(prhs[0]);
  n = mxGetM(prhs[0]);
  k = mxGetN(prhs[0]);
  
  B = mxGetPr(prhs[1]);
  foo = mxGetM(prhs[1]);
  if (foo != k) 
    mexErrMsgTxt("Matrix B must have same number of rows as A has columns.");
  m = mxGetN(prhs[1]);
    
  /* Output argument C */
  plhs[0] = mxCreateDoubleMatrix(n, m, mxREAL);
  C = mxGetPr(plhs[0]);
  

  /* The meat */
  /* Everything is in column-wise order, so let's proceed in that order, too*/
  for (j=0; j < m; j++) {
    for (i=0; i < n; i++) {
      for (l=0; l < k; l++) {
	if (A[element(n, i, l)] && B[element(k, l, j)]) {
	  C[element(n, i, j)] = 1.0;
	  break;
	}
      }
    }
  }

}

