#include <string.h>
#include <stdint.h>
#include "mex.h"
#include "matrix.h"

#define element(rows, row, col) (rows*col + row)
#define MAX(a,b) ((a>b)?a:b)
#define MIN(a,b) ((a<b)?a:b)

  
/*
 * C = bprod(A, B)
 *
 * Returns the Boolean product of A and B, which must be of type 'uint8'.
 *
 */ 

void 
mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  mwSize n, m, k, i, j, l, colsOfA, rowsOfB;
  uint8_t * restrict A, * restrict B;
  double * restrict C;


  /* Check for input and output argument numbers */
  if (nrhs != 2 && nrhs != 3) 
    mexErrMsgTxt("Two or three input arguments, A, B, [and k,] required.");
  if (nlhs > 1)
    mexErrMsgTxt("Too many output arguments.");
  
  /* Check and load input matrices. */
  if (!mxIsUint8(prhs[0]))
    mexErrMsgTxt("Both input matrices must be of type 'uint8'; the first is not.");
  A = (uint8_t *)mxGetData(prhs[0]);
  n = mxGetN(prhs[0]);
  colsOfA = mxGetM(prhs[0]);
  
  if (!mxIsUint8(prhs[1]))
    mexErrMsgTxt("Both input matrices must be of type 'uint8'; the second is not.");
  B = (uint8_t *)mxGetData(prhs[1]);
  rowsOfB = mxGetM(prhs[1]);
  if (nrhs == 2 && rowsOfB != colsOfA) 
    mexErrMsgTxt("Matrix B must have same number of rows as A has columns.");
  m = mxGetN(prhs[1]);
    
  /* If separate k is given */
  if (nrhs == 3) {
    k = (mwSize)mxGetScalar(prhs[2]);
    if (k <= 0)
      mexErrMsgTxt("k must be positive integer.");
    if (k > colsOfA || k > rowsOfB)
      mexErrMsgTxt("k cannot be larger than the number of columns of A or the" 
		   "number of rows of B.");
  } else {
    k = rowsOfB; /* == colsOfA */
  }

  /* Output argument C */
  plhs[0] = mxCreateDoubleMatrix(n, m, mxREAL);
  C = mxGetPr(plhs[0]);
  

  /* The meat */
  /* Everything is in column-wise order, so let's proceed in that order, too*/
  for (j=0; j < m; j++) {
    for (i=0; i < n; i++) {
      for (l=0; l < k; l++) {
	if (A[element(colsOfA, l, i)] && B[element(rowsOfB, l, j)]) {
	  C[element(n, i, j)] = 1.0;
	  break;
	}
      }
    }
  }

}

