function [c, C, X] = cover(A, M, i, wp);
% COVER - Compute the cover function.
% Inputs:
% A          The data matrix
% M          Bit mask for already covered elements
% i          Index of the column (scalar) OR the column itself (vector).
%
% c = COVER(A, M, i) returns the cover-value
% [c, C] = COVER(A, M, i) returns the cover-value and an updated
%   bit mask.
% [c, C, X] = COVER(A, M, i) returns also the column indices where
% the i'th column was used.
% c = COVER(A, M, i, wp) values covered 1s by wp.
  
  if nargin == 3,
    wp = 1;
  end;

  [n,m] = size(A);
  [x,y] = size(M);
  if n ~= x || m ~= y,
    print 'Wrong sized matrices!';
    return;
  end
  
  
  if isscalar(i),
    v = A(:,i);
  elseif isvector(i),
    v = i;
  else
    error('i must be either vector or scalar');
  end;
  I = repmat(v, 1, m);
  P = wp * sum(double(M == 0 & A == 1 & I == 1));
  N = sum(double(M == 0 & A == 0 & I == 1));
  c = sum(max(0, P - N));
  X = double(P > N);
  C = max(M, min(1, v*X));
  
