function [Y, ERR] = iterX(D,C,X,verb)
% ITERX - Improves X using iteration.
% Y = ITERX(D, C, X) returns updated X that has converged to some
% local optima. If C is a vector, it is assumed to give A's column
% indices constituting the matrix C.
%
% [Y, ERR] = ITERX(D,C,X) also returns the error.
%
% Y = ITERX(D, C, X, verb) controls the verbosity level.
  
  MAXITER = 300;
  
  error(nargchk(3, 4, nargin));
  
  [n,m]=size(D);
  
  if nargin < 4,
    verb = 0;
  end;
  
  if isvector(C),
    idx = C;
    C = D(:,idx);
    c = length(idx);
  else,
    [foo, c] = size(C);
  end;
  
  err = bnorm(D, C, X);
  
  bestErr = Inf;
  iter = 1;

  
  while err < bestErr && iter < MAXITER,
    if verb == 1,
      fprintf('\b\b\b%3i', iter);
    elseif verb > 1,
      fprintf(['\r                                                        '...
               '\r%i: %d    '], iter, err);
    end;
    bestErr = err;
    bestX = X;
    iter = iter + 1;
    bestIterError = err;
    
    for i=1:c,
      if verb > 1,
        fprintf('\b\b\b%3i        ', i);
      end;
      %bestx = i;
      tmpX = X;
      tmpX(i,:) = zeros(1,m);
      M = min(1, C*tmpX);
      [foo1, foo2, tmpx] = cover(D, M, C(:,i));
      %if isvector(idx),
      %  [foo1, foo2, tmpx] = cover(D, M, idx(i));
      %else,
      %  [foo1, foo2, tmpx] = cover(D, M, C(:,i));
      %end
      tmpX(i,:) = tmpx;
      tmpErr = bnorm(D, C, tmpX);
      
      %fprintf(['tmpErr = ', int2str(tmpErr), ' and bestIterError = ', ...
      %         int2str(bestIterError), '\n']);
      
      if tmpErr < bestIterError,
        bestIterError = tmpErr;
        bestX = tmpX;
      end;
      
      if verb > 1,
        fprintf('\b\b\b\b\b\b\b\b');
      end;

    end;
        
    err = bnorm(D, C, bestX);
    if  err < bestErr,
      X = bestX;
    end;
  end;
 
  if verb > 1,
    fprintf('\n');
  end;

  Y = X;
  ERR = bnorm(D, C, X);

