function [k, L, LE, B, C] = pandamdl(A, eM, r, v)
% PANDAMDL -- Lets PANDA choose optimum k and compute MDL to it
%
% USAGE
%           [k, L] = PANDAMDL(A)           returns k selected by PANDA
%                                            and the MDL score for that
%       [k, L, LE] = PANDAMDL(A)           returns also the error part of MDL
% [k, L, LE, B, C] = PANDAMDL(A )          returns also the factor matrices
%           [k, L] = PANDAMDL(A, eM)       uses error measure eM with
%                                            DESCRIPTION_LENGTH (default: all)
%           [k, L] = PANDAMDL(A, eM, r)    uses r random rounds in PANDA
%                                            (default: 0)
%           [k, L] = PANDAMDL(A, eM, r, v) sets verbosity level (default: 0)
%           
% See also: DESCRIPTION_LENGTH, PANDA

error(nargchk(1, 4, nargin));
if nargin < 2, 
    eM = {'naiveXor', 'typedXor', 'naiveIndex', 'naiveFactor', 'DtMnaiveXor', 'DtMtypedXor'};
end
if nargin < 3,
    r = 0;
end
if nargin < 4,
    v = 0;
end

% RUN PANDA
if v > 0, fprintf('STARTING PANDA\n==================\n'); end

[B, C] = panda(A, -1, r, v);
     
if v > 0, fprintf('\n==================\nPANDA ENDS\n'); end

k = size(B, 2);

% DEBUG
%fprintf('B is %i-by-%i C is %i-by-%i and k is %i\n', size(B), size(C), k);
%fprintf('A is %i-by-%i\n', size(A));

if v > 0, fprintf('Computing description length... '); end
[L, LE] = description_length(A, B, C, k, eM);
if v > 0, fprintf('done!\n'); end
