function [ tcost, Xcomp ] = tcostBinary(Xtrain, Xtest, u, z, Xcomp)
% computes Hamming distance between a trained Boolean decomposition and a
% hold out test-set
% repeatedly call this function with different Xtrain, Xtest to get an
% estimate of the expected generalization error

%{
input:
    Xtrain:     binary matrix used to train decomposition, rows are objects
                and columns are features
    Xtest:      binary matrix used for testing. Same format as Xtrain.
    u:          binary factor-to-feature matrix (role-permission assignment
                in role mining context)
    z:          binary object-to-factor matrix (user-role assignment)
    Xcomp:      optional argument. Xcomp is the pairwise costs between 
                objects of  Xtest and Xtrain. If Xcomp is not provided 
                from externally, it will be internally computed 
                (costs a lot)

output:         fraction of wrongly predicted bits in Xtest
%}

[Ntrain D] = size(Xtrain);
Ntest = size(Xtest, 1);

% pairwise comparison (costliest part of computation, provide Xcomp if you
% know it already)
if ~exist('Xcomp', 'var') || isempty(Xcomp)
    Xcomp = single(zeros(Ntrain, Ntest));
    for i=1:Ntrain
        tmp = repmat(Xtrain(i,:), [Ntest 1]);
        Xcomp(i,:) = sum(abs(tmp-Xtest), 2);
    end
    Xcomp = Xcomp/D;
end

% find nearest neighbors
[ forgetme, matchPerm ] = min(Xcomp,[],1);

% aligns objects of test set with objects of training set. Multiple
% testing objects can be mapped to one training object. And some test
% objects can be ignored. This could be improved (by a return-mapping for
% instance) but we had no problem in the experiments so far.
z = z(matchPerm,:);

% l1-norm for deviation between prediction and test data
tcost = sum(sum(abs( Xtest - single((z*u)>0) )));

% ..per matrix entry
tcost = tcost /Ntest/D;
